=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-17 10:50:39 +0000
+++ bzrlib/branch.py	2005-10-17 11:31:05 +0000
@@ -43,6 +43,7 @@
 from bzrlib.store.compressed_text import CompressedTextStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.weave import WeaveStore
+from bzrlib.testament import Testament
 import bzrlib.transactions as transactions
 from bzrlib.transport import Transport, get_transport
 import bzrlib.xml5
@@ -1288,8 +1289,14 @@
         if revno < 1 or revno > self.revno():
             raise InvalidRevisionNumber(revno)
         
-        
-        
+    def sign_revision(self, revision_id, gpg_strategy):
+        self.lock_write()
+        try:
+            plaintext = Testament.from_revision(self, revision_id).as_short_text()
+            self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
+                                    revision_id, "sig")
+        finally:
+            self.unlock()
 
 
 class ScratchBranch(_Branch):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 10:50:39 +0000
+++ bzrlib/builtins.py	2005-10-17 11:31:05 +0000
@@ -1497,20 +1497,13 @@
         b = Branch.open_containing('.')
         gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
         if revision_id is not None:
-            self.sign_one(gpg_strategy, b, revision_id)
+            b.sign_revision(revision_id, gpg_strategy)
         elif revision is not None:
             for rev in revision:
                 if rev is None:
                     raise BzrCommandError('You cannot specify a NULL revision.')
                 revno, rev_id = rev.in_history(b)
-                self.sign_one(gpg_strategy, b, rev_id)
-
-    def sign_one(self, gpg_strategy, b, revision_id):
-        import bzrlib.testament as testament
-        from cStringIO import StringIO
-        plaintext = testament.Testament.from_revision(b, revision_id).as_short_text()
-        b.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), revision_id, "sig")
-    
+                b.sign_revision(rev_id, gpg_strategy)
 
 
 # these get imported and then picked up by the scan for cmd_*

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-15 11:38:29 +0000
+++ bzrlib/commit.py	2005-10-17 11:31:05 +0000
@@ -77,6 +77,7 @@
                            HistoryMissing,
                            ConflictsInTree
                            )
+import bzrlib.gpg as gpg
 from bzrlib.revision import Revision
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
@@ -145,12 +146,16 @@
             working inventory.
     """
     def __init__(self,
-                 reporter=None):
+                 reporter=None,
+                 config=None):
         if reporter is not None:
             self.reporter = reporter
         else:
             self.reporter = NullCommitReporter()
-
+        if config is not None:
+            self.config = config
+        else:
+            self.config = None
         
     def commit(self,
                branch, message,
@@ -194,14 +199,16 @@
         else:
             self.timestamp = long(timestamp)
             
-        config = bzrlib.config.BranchConfig(self.branch)
+        if self.config is None:
+            self.config = bzrlib.config.BranchConfig(self.branch)
+
         if rev_id is None:
-            self.rev_id = _gen_revision_id(config, self.timestamp)
+            self.rev_id = _gen_revision_id(self.config, self.timestamp)
         else:
             self.rev_id = rev_id
 
         if committer is None:
-            self.committer = config.username()
+            self.committer = self.config.username()
         else:
             assert isinstance(committer, basestring), type(committer)
             self.committer = committer
@@ -315,6 +322,8 @@
         serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
         self.branch.revision_store.add(rev_tmp, self.rev_id)
+        if self.config.signature_needed():
+            self.branch.sign_revision(self.rev_id, gpg.GPGStrategy(self.config))
         mutter('new revision_id is {%s}', self.rev_id)
 
     def _remove_deleted(self):

=== modified file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-16 14:51:21 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-17 11:31:05 +0000
@@ -20,11 +20,21 @@
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.commit import Commit
+from bzrlib.config import BranchConfig
 from bzrlib.errors import PointlessCommit, BzrError
 
 
 # TODO: Test commit with some added, and added-but-missing files
 
+class MustSignConfig(BranchConfig):
+
+    def signature_needed(self):
+        return True
+
+    def gpg_signing_command(self):
+        return ['cat', '-']
+
+
 class TestCommit(TestCaseInTempDir):
 
     def test_simple_commit(self):
@@ -245,3 +255,22 @@
         self.assertEqual('1', inv['file1id'].revision)
         # FIXME: This should raise a KeyError I think, rbc20051006
         self.assertRaises(BzrError, inv.__getitem__, 'file2id')
+
+    def test_signed_commit(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            commit.Commit(config=MustSignConfig(branch)).commit(branch, "base",
+                                                      allow_pointless=True,
+                                                      rev_id='B')
+            self.assertEqual(Testament.from_revision(branch,'B').as_short_text(),
+                             branch.revision_store.get('B', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-09 22:43:56 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-17 11:31:05 +0000
@@ -141,6 +141,15 @@
         # list should be cleared when we do a commit
         self.assertEquals(b.pending_merges(), [])
 
+    def test_sign_existing_revision(self):
+        import bzrlib.gpg
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        from bzrlib.testament import Testament
+        branch.sign_revision('A', bzrlib.gpg.LoopbackGPGStrategy(None))
+        self.assertEqual(Testament.from_revision(branch, 'A').as_short_text(),
+                         branch.revision_store.get('A', 'sig').read())
+
 
 class TestRemote(TestCaseWithWebserver):
 

